<?php

/*
* FothoFax - A Simple FiloFax replacement
* Copyright (C) 2005 Tom Fotherby
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


//This include file gathers data and contains global variables and config settings.
require('fothoFaxConfig.inc');


//////////////////////////////////////////////////////////////////////////
// This is the main function - It draws the entire calendar.
function drawCalendar($numWeeksBack,$numWeeksForward)
{
  $count = 0;

  //Calculate how many days from today we start the calendar (based on input params)
  if (Date("w") == 0)   // If it is Sunday.
   $when2Start = -6 - (7*$numWeeksBack);
  else
   $when2Start = -(Date("w")-1) - (7*$numWeeksBack);  //days since monday minus a week.

  //How many days do we want the calendar to show?
  $when2End = $when2Start + (7*$numWeeksForward) + (7*$numWeeksBack);

  //Output the first row of the calendar
  echo "<table class=\"main\" cellpadding=\"0\" border=\"1\" cellspacing=\"0\">\n";
  echo "<tr align=\"center\">\n";
  echo  "<td class=\"title\">" . getCaptionBoxInfo() . "</td>\n";
  echo  "<td colspan=\"7\" align=\"center\" class=\"title\">";
  echo  ($numWeeksBack+$numWeeksForward) . " week calendar</td>\n";
  echo "</tr>\n";

  //Output the row containing the days of the week
  echo "<tr align=\"center\"><th class=\"daysofweek\">&nbsp;</th><th class=\"daysofweek\">M</th><th class=\"daysofweek\">T</th><th class=\"daysofweek\">W</th><th class=\"daysofweek\">T</th><th class=\"daysofweek\">F</th><th class=\"daysofweek\">S</th><th class=\"daysofweek\">S</th>\n";

  //Loop through from the start day to the end day showing date for each day
  for ($i=$when2Start; $i<$when2End; $i++)
  {
    $curDay = strtotime("+".$i." days");
    $cssText = "";

     if (strftime("%d",$curDay) <= 7)
	$cssText = "OnNewMonth";
     else if (strftime("%d",$curDay) == date("t",$curDay))
	$cssText = "OnLastDayOfMonth";

    if ($count%7 == 0) //If Start of week -> Output the month name in col 1
      echo "<tr align=\"center\"><td class=\"month$cssText\">" . strftime("%b",$curDay) . "</td>\n";

    if ($i == 0)
     $cssText = "today$cssText";
    else if ($count%7 >= 5)
     $cssText = "weekend$cssText";
    else
     $cssText = "weekday$cssText";

    echo "<td class=\"$cssText\">". getDayInfo($curDay,$cssText) . "</td>\n";

    if ($count%7 == 6)   //End of week.
      echo "</tr>\n";

    $count++;
  }

  echo "</table>\n";
}

//////////////////////////////////////////////////////////////////////////
function getCaptionBoxInfo()
{
  global $bdayFileEditURL,$eventFileEditURL,$version,$bdayArry,$evntArry;

  return "<a class=\"title\" href=\"javascript:void(0);\" onmouseover=\"return overlib('<ul><li>Version: $version<li>Author: <a href=\'http://www.tomfotherby.com\'>Tom Fotherby</a><li>Software License: <a href=\'http://www.gnu.org/copyleft/gpl.html\'>GPL</a><li>$bdayFileEditURL. Number: ". sizeof($bdayArry) ."<li>$eventFileEditURL. Number: ". sizeof($evntArry) ."</ul>',CAPTION,'About FothoFax',STICKY);\" onmouseout=\"return nd();\">?</a>";
}


//////////////////////////////////////////////////////////////////////////
// Input is a timeStamp, output is html code describing the day. e.g. birthday and events for that day.
function getDayInfo($dateToQuery,$cssText)
{
  global $imagesFile,$bdayArry,$ddayArry,$evntArry;
  global $anniversary_julianDay,$anniMonth,$anniDay,$anniversaryText,$anniDayReminder;

 $addedBdayInfo  = "";
 $addedEventInfo = "";
 $addedRemembranceInfo = "";

 $dateToTest      = strftime("%d/%b", $dateToQuery);
 $evntDateToTest  = strftime("%d/%b/%y", $dateToQuery);
 $date_MonDayYear = explode("/",strftime("%m/%d/%Y", $dateToQuery));
 $today_Mon       = $date_MonDayYear[0];
 $today_Day       = $date_MonDayYear[1];
 $today_Year      = $date_MonDayYear[2];
 $today_jd        = gregoriantojd($today_Mon,$today_Day,$today_Year);

 // See if there's any birthdays on this day
 if ($bdayArry[$dateToTest]) {
    foreach ($bdayArry[$dateToTest] as $person) {
       $currentName = $person["Name"];
       $age = $person["Age"];
       if ($person["Dday"] == "")
	$addedBdayInfo .= "<li>$currentName\'s $age". getSuffix($age) . " birthday";
       else
        $addedBdayInfo .= "<li>It would have been $currentName\'s $age". getSuffix($age) . " birthday";
    }
 }

 //Make sure leap babies get a birthday every year!
 if (!date("L") and               // If it's not leap year
     $dateToTest == "28/Feb" and  // and it's the day before the usual leap day
     $bdayArry["29/Feb"]) {       // and there are people with birthdays on the 29th.
    foreach ($bdayArry["29/Feb"] as $person) {
       $currentName = $person["Name"];
       $age = $person["Age"];
       if ($person["Dday"] == "")
	 $addedBdayInfo .= "<li>$currentName\'s $age". getSuffix($age) . " birthday";
       else
         $addedBdayInfo .= "<li>It would have been $currentName\'s $age". getSuffix($age) . " birthday";
       $addedBdayInfo .= " (because it\'s not a leap year this year)";
    }
 }

 // See if there's any remembrance days on this day
 if ($ddayArry[$dateToTest]) {
    foreach ($ddayArry[$dateToTest] as $person) {	
       $addedRemembranceInfo .= "<li>".$person["Name"]." died on " . $person["Dday"];
    }
 }

 // See if there's any events on this day
 if ($evntArry[$evntDateToTest]) {
    foreach ($evntArry[$evntDateToTest] as $event) { 
       $addedEventInfo .= "<li>" . makeStringSafe($event);
    }
 }

 // See if this day is an anniversay
 if (($today_jd-$anniversary_julianDay)%$anniDayReminder == 0)
   $addedEventInfo .= "<li>It was " . ($today_jd-$anniversary_julianDay) . " days since $anniversaryText!";
 
 if ($anniMonth == $today_Mon && $anniDay == $today_Day)
   $addedEventInfo .= "<li>$anniversaryText today!";

  // Now that we have collected info for this day. Format how it is displayed:

  $addedInfo = "";  // The total HTML description of the day
  $popupText = "";  // var to store the text displayed in the popup box for this day
  $iconFile  = "";  // The name of the icon image file for the day
  $linkClass = "class=\"$cssText\"";  // Used to un-boldify hyperlinks

  if ($addedBdayInfo != "")
  {
	$popupText .= "<b>Birthdays</b><br><ul>$addedBdayInfo</ul>";
	$iconFile  .= "candle";
  }

  if ($addedRemembranceInfo != "")
  {
	$popupText .= "<b>Remembrances</b><br><ul>$addedRemembranceInfo</ul>";
	$iconFile  .= "poppy";
  }

  if ($addedEventInfo != "")
  {
	$popupText .= "<b>Events</b><br><ul>$addedEventInfo</ul>";
  }

  if ($popupText == "")
  {
    // Nothing is happening on this day
    $addedInfo = "<font color=\"gray\">" . strftime("%d",$dateToQuery) . "</font>";
  } else {
    // This day has something going on in it
    if ($iconFile != "")
    {
      // An icon is used on this day (i.e. a candle or poppy)
      $addedInfo  ="<img border=\"0\" alt=\"$iconFile.gif\" src=\"$imagesFile/$iconFile.gif\">";
    }
    if ($addedEventInfo == "")
    {
      // There is no event this day so make sure the day number isn't in bold
      $linkClass = "class=\"notBold\"";
    }
    $addedInfo = "<a $linkClass href=\"javascript:void(0);\" onmouseover=\"return overlib('$popupText',CAPTION,'On $dateToTest:');\" onmouseout=\"return nd();\">" . strftime("%d",$dateToQuery) . "$addedInfo</a>";

  }

  return $addedInfo; 
}


//////////////////////////////////////////////////////////////////////////
// This prints out all the upcoming events from today till $numDaysAhead
function OutputUpcomingEvents($numDaysAhead)
{
 global $evntArry;

 //Output upcoming events (From Today to $numDaysAhead):
 for ($i=0; $i<$numDaysAhead; $i++) 
 {
  $dateToTest = strftime ("%d/%b/%y", strtotime("+" . $i . " days"));
  // See if there's any events on this day
  if ($evntArry[$dateToTest]) {
    foreach ($evntArry[$dateToTest] as $event) {
       print "On $dateToTest: $event<br>\n";
    }
  }
 }
}

//////////////////////////////////////////////////////////////////////////
// This prints out the next n events
function OutputEvents($num)
{
  global $evntArry;

  $eventCount    = 0;
  $numDaysAhead  = 0;
  $curTime       = strtotime("today");
  $lastEventTime = getLastEventTime();

  //Output upcoming events (Start from Today and work forwards):
  while ($eventCount < $num and $curTime <= $lastEventTime)
  {
    $curTime    = strtotime("+" . $numDaysAhead . " days");
    $dateToTest = strftime ("%d/%b/%y", $curTime);

    // See if there's any events on this day
    if ($evntArry[$dateToTest]) {
      foreach ($evntArry[$dateToTest] as $event) {
	if ($numDaysAhead == 0)
	  print "$dateToTest (<font color=\"red\"><b>today</b></font>): $event<br>\n";
	else if ($numDaysAhead == 1)
	  print "$dateToTest (<font color=\"red\"><b>tomorrow</b></font>): $event<br>\n";
	else
	  print "$dateToTest ($numDaysAhead days): $event<br>\n";
	$eventCount++;
      }
    }
    $numDaysAhead++;
 }
}

//////////////////////////////////////////////////////////////////////////
// This prints out all the upcoming birthdays from today till $numDaysAhead
function OutputUpcomingBirthdays($numDaysAhead)
{
  global $bdayArry;

  //Output upcoming birthdays (From Today to $numDaysAhead):
  for ($i=0; $i<$numDaysAhead; $i++) 
  {
	$dateToTest = strftime ("%d/%b", strtotime("+" . $i . " days"));
  	// See if there's any birthdays on this day
  	if ($bdayArry[$dateToTest]) {
    	 foreach ($bdayArry[$dateToTest] as $person) {
       	  print "On $dateToTest, it's " . $person["Name"] . "'s birthday. They are "  . $person["Age"]  . ".<br>\n";
    	 }
  	}
  }
}

//////////////////////////////////////////////////////////////////////////
// This prints out the next n birthdays
function OutputBirthdays($num)
{
  global $bdayArry;

  $bdayCount = 0;
  $numDaysAhead = 0;

  //Output upcoming birthdays (Start from Today and work forwards a year):
  while ($bdayCount < $num and $numDaysAhead < 365)
  {
     $dateToTest = strftime ("%d/%b", strtotime("+" . $numDaysAhead . " days"));
     // See if there's any birthdays on this day
     if ($bdayArry[$dateToTest]) {
    	 foreach ($bdayArry[$dateToTest] as $person) {
	   if ($numDaysAhead == 0)
	     print "$dateToTest: It's " . $person["Name"] . "'s birthday <font color=\"red\"><b>today</b></font>. They are "  . $person["Age"]  . ".<br>\n";
	   else if ($numDaysAhead == 1)
	     print "$dateToTest: It's " . $person["Name"] . "'s birthday <font color=\"red\"><b>tomorrow</b></font>. They are "  . $person["Age"]  . ".<br>\n";
	   else
	     print "$dateToTest: It's " . $person["Name"] . "'s birthday in $numDaysAhead days. They are "  . $person["Age"]  . ".<br>\n";
           $bdayCount++;
     	 }
  	}
	$numDaysAhead++;
  }
}



//////////////////////////////////////////////////////////////////////////
////////////////  U t i l i t y   F u n c t i o n s  /////////////////////
//////////////////////////////////////////////////////////////////////////



//////////////////////////////////////////////////////////////////////////
// This loops through the eventArray Keys looking for the date furthest in the future.
function getLastEventTime()
{
  global $evntArry;

  $months = array('Jan'=>'1','Feb'=>'2','Mar'=>'3','Apr'=>'4','May'=>'5','Jun'=>'6', 'Jul'=>'7', 'Aug'=>'8', 'Sep'=>'9', 'Oct'=>'10', 'Nov'=>'11', 'Dec'=>'12');
  $lastDate = strtotime("today");

  foreach ($evntArry as $eventDate => $eventArray) {

    $eventDay   = substr($eventDate,0,2);
    $eventMonth = $months[substr($eventDate,3,3)];
    $eventYear  = substr($eventDate,7,2);
    $curDate    = mktime(0,0,0,$eventMonth,$eventDay,$eventYear);

    if ($curDate > $lastDate)
      $lastDate = $curDate;
  }
  return $lastDate;
}

//////////////////////////////////////////////////////////////////////////
// This utility function takes out any characters that causes Overlib to break
// Overlib is the javascript popup library (written by Erik Bosrup) that I use.
function makeStringSafe($theString)
{
  $theString = str_replace("'", "\'", $theString);      //Escape single quote (but can't use html!)
  $theString = str_replace("\"", "&quot;", $theString); //Replace double-quotes with html.
  return $theString;
}

//////////////////////////////////////////////////////////////////////////
// Example input is "22". Output would be "nd".
function getSuffix($number)
{
   switch ( $number % 10 )
   {
     case '1': return 'st';
     case '2': return 'nd';
     case '3': return 'rd';
     default:  return 'th';
   }
}

?>